import common
from common import AddonSettings, KODI_ADDON
from overlay import Overlay
from smarty_api.user_api import UserApiClient
from splash_window import show_splash_window
from tv_player import TVPlayer


class Authenticator(object):
    def __init__(self, client=None):
        self._client = UserApiClient() if client is None else client
        self.onAuthStarted = None
        self.onAuthEnded = None

        self.isAuthChanged = False
        self.authChangedByClient = False

    @staticmethod
    def checkAuthChanged():
        common.log_debug("checkAuthChanged", 'app')
        abonement = AddonSettings.get_setting(AddonSettings.ABONEMENT)
        password = AddonSettings.get_setting(AddonSettings.PASSWORD)
        abonementOld = AddonSettings.get_setting(AddonSettings.ABONEMENT_OLD)
        passwordOld = AddonSettings.get_setting(AddonSettings.PASSWORD_OLD)

        if abonement != abonementOld or password != passwordOld:
            AddonSettings.set_setting(AddonSettings.ABONEMENT_OLD, abonement)
            AddonSettings.set_setting(AddonSettings.PASSWORD_OLD, password)
            return True
        return False

    def checkUserSettings(self):
        if not AddonSettings.check_setting_non_empty(AddonSettings.ABONEMENT):
            AddonSettings.clear_setting(AddonSettings.PASSWORD)
            common.open_settings()
            self.authChangedByClient = True
            if not AddonSettings.check_setting_non_empty(AddonSettings.ABONEMENT):
                dialog = common.GuiDialog(common.get_localized_string(32403), level=common.GuiDialog.LEVEL_ERROR)
                dialog.show()
                return False
        return True

    @staticmethod
    def clear_credentials():
        AddonSettings.clear_setting(AddonSettings.ABONEMENT)
        AddonSettings.clear_setting(AddonSettings.PASSWORD)
        AddonSettings.clear_setting(AddonSettings.ABONEMENT_OLD)
        AddonSettings.clear_setting(AddonSettings.PASSWORD_OLD)
        Authenticator.clear_auth_key()

    @staticmethod
    def clear_auth_key():
        AddonSettings.clear_setting(AddonSettings.AUTHKEY)

    def auth(self):
        common.log_debug("Authentication", 'app')
        Authenticator.clear_auth_key()
        self._client = UserApiClient()
        if self._client is None:
            return -20

        error = self._client.auth()
        if error == 0:
            AddonSettings.set_setting(AddonSettings.AUTHKEY, self._client.user.authkey)
        elif self.authChangedByClient:
            self.clear_credentials()
            self.authChangedByClient = False
        return error

    def logout(self):
        common.log_debug("logout", 'app')

        if self._client is None:
            return False

        error = self._client.logout()
        Authenticator.clear_credentials()

        if error == 0:
            return True
        return False

    @property
    def client(self):
        return self._client


class App(object):
    def __init__(self):
        self._client = None

        self._player = TVPlayer()
        self._player.stop()

        Authenticator.clear_auth_key()

        skin_name = AddonSettings.get_skin_name()
        common.log_debug("Start plugin, set skin: %s" % skin_name, 'App')
        self._overlay = Overlay("smarty.tv.overlay.xml", KODI_ADDON.getAddonInfo('path'),
                                skin_name, player=self._player)

        self._player.onTVStartedAction = self._overlay.startedAction
        self._player.onTVStoppedAction = self._overlay.stoppedAction

        self._player.onPlayBackPausedAction = self._overlay.pausedAction
        self._player.onPlayBackResumedAction = self._overlay.resumedAction
        self._player.onPlayBackStartedAction = self._overlay.channelChangeEndsAction
        self._player.onChannelChangedAction = self._overlay.channelChangeStartAction

        self._player.onArchiveEnabledAction = self._overlay.archiveModeEnabledAction
        self._player.onArchiveTimeSetAction = self._overlay.archiveTimeSetAction

        self._overlay.onLogoutAction = self.logout
        self._overlay.onAuthAction = self.auth
        self._overlay.onExitAction = self.exit

        self._overlay.onOverlayStarted = self.startOverlayAction
        self._overlay.onSettingsChanged = self.settingsChangedAction

        self.nextRefreshTimer = None
        self.progressBarLoop = None
        self.currentPlaylist = []
        self.progressBegin = 0
        self.progressEnd = 0

    def start(self):
        common.log_debug("start", 'app')
        self._overlay.doModal()
        del self._overlay

    def startOverlayAction(self):
        common.log_debug("startOverlay", 'app')
        self.auth()

    def settingsChangedAction(self):
        common.log_debug("settingsChanged", 'app')
        Authenticator.checkAuthChanged()

    def exit(self):
        pass

    def auth(self):
        common.log_debug("Authentication", 'app')

        authenticator = Authenticator()
        if not authenticator.checkUserSettings():
            return

        spl = show_splash_window(common.get_localized_string(32400))
        error = authenticator.auth()

        self._client = authenticator.client

        if error != 0:
            common.close_splash_window(spl)
            if error == 1:
                message = common.get_localized_string(32004)
            elif error == UserApiClient.REQUEST_TIME_OUT:
                message = common.get_localized_string(32005)
            elif error == UserApiClient.AUTH_APIKEY_CHECK_ERROR:
                message = common.get_localized_string(32006)
            elif 600 > error >= 500 or error == UserApiClient.AUTH_AUTHKEY_GENERATION_ERROR:
                message = common.get_localized_string(32007)
            elif error == -20:
                message = common.get_localized_string(32004)
            else:
                message = common.get_localized_string(32008)
            dialog = common.GuiDialog(message, level=common.GuiDialog.LEVEL_ERROR)
            dialog.show()
        else:
            self._overlay.onAuth()
            common.close_splash_window(spl)
        return error

    def logout(self):
        authenticator = Authenticator()
        authenticator.logout()
        self._overlay.onLogout()
