import common
import threading
import time


class Event(object):
    def __init__(self, time, function, id):
        self.time = time
        self.function = function
        self.id = id

    def __le__(self, other):
        return self.time <= other.time

    def __lt__(self, other):
        return self.time < other.time


class ProgressLoop(threading.Thread):
    def __init__(self):
        self._nextRefresh = -1
        self._begin = 0
        self._end = 0
        self._started = False

        self._onLoopStopped = None
        self._onTimeChanged = None
        self._onTimerWork = None
        self._onTimeEnds = None

        self._events = []

        self._work_enabled = False
        self._timeLock = threading.Lock()
        self._defaultListRefreshTime = 300.0  # in seconds
        self._defaultWorkTime = 1.0  # in seconds
        self._currentTimeDelta = 0.0
        threading.Thread.__init__(self)

    def setLoopStopped(self, function):
        self._onLoopStopped = function

    def setOnWorkTimeChanged(self, function):
        self._onTimeChanged = function

    def setTimerWork(self, function):
        self._onTimerWork = function

    def setTimeEnds(self, function):
        self._onTimeEnds = function

    def setWorkEnabled(self, isEnabled):
        common.log_debug("Set work enabled: %s" % str(isEnabled), "progressLoop.setWorkEnabled")
        self._work_enabled = isEnabled

    def stop(self):
        common.log_info("Stop loop", "progressLoop.start")
        self._started = False
        if self._onLoopStopped:
            self._onLoopStopped(self, ())

    def setTime(self, begin, end):
        common.log_info('Set time: %d-%d' % (begin, end), "progressLoop.setTime")

        with self._timeLock:
            if begin == 0:
                self._work_enabled = False
                return
            self._work_enabled = True
            self._begin = begin
            self._end = end

        self._work(time.time())
        if self._onTimeChanged:
            self._onTimeChanged(self, (begin, end))

    def _removeEventUnsync(self, event_id):
        it = -1
        for i in range(len(self._events)):
            if self._events[i].id == event_id:
                it = i
                break
        if it >= 0:
            self._events.pop(it)

    def addEvent(self, event):
        common.log_debug("Add event: %s, id: %s" % (str(event.function), event.id), "progressLoop.addEvent")
        if event.time < time.time():
            return

        with self._timeLock:
            self._removeEventUnsync(event.id)
            for i in range(len(self._events)):
                if event < self._events[i]:
                    self._events.insert(i, event)
                    break
            else:
                self._events.append(event)

    def removeEvent(self, event_id):
        with self._timeLock:
            self._removeEventUnsync(event_id)

    def clearTime(self):
        common.log_info('Clear time', "progressLoop.setTime")
        self.setTime(0, 0)

    def _work(self, currentTime):
        with self._timeLock:
            if 0 < self._begin <= currentTime <= self._end:
                self._onTimerWork(self, (self._begin, currentTime, self._end))
            else:
                self._onTimerWork(self, (0.0, 0.0, 0.0))

    def run(self):
        common.log_info("Progress loop thread started", "progressLoop.run")
        self._started = True
        monitor = common.KODI_MONITOR

        while self._started:
            if monitor.abortRequested():
                return

            currentTime = time.time()
            while True:
                with self._timeLock:
                    if len(self._events) == 0 or self._events[0].time >= currentTime:
                        break
                    event = self._events[0]
                    self._events.pop(0)

                common.log_info("Call event %s" % event.id, "progressLoop.run")
                event.function(self, None)
            if self._work_enabled:
                if 0 < self._end < currentTime:
                    self._onTimeEnds(self, None)
                else:
                    self._work(currentTime)
            time.sleep(self._defaultWorkTime)
        common.log_info("Progress loop thread stopped", "progressLoop.run")

    @property
    def started(self):
        return self._started


class RefreshableTimer(threading.Thread):
    def __init__(self, action, eventTime):
        self._action = action
        self._timer = eventTime + time.time()
        self._timeLock = threading.Lock()
        self._started = False
        threading.Thread.__init__(self)

    def refreshTime(self, eventTime):
        with self._timeLock:
            self._timer = eventTime + time.time()

    def stop(self):
        self._started = False

    def run(self):
        monitor = common.KODI_MONITOR

        self._started = True
        while self._started:
            if monitor.abortRequested():
                return

            with self._timeLock:
                if time.time() >= self._timer:
                    self._action()
                    self._started = False
                    return
            time.sleep(0.1)

    @property
    def started(self):
        return self._started
