from time import time

from smarty_base_api import BaseSmartyApiClient, SmartyAPIURL
from resources.lib.common import log_info, log_debug, log_error, log_warning
from program_api import Program


class ChannelField:
    ID = 'id'
    NAME = 'name'
    STREAM_URL = 'url'
    ICON_URL = 'icon'
    PROGRAM_NAME = 'program_name'
    PROGRAM_DESCRIPTION = 'program_description'
    PROGRAM_BEGIN_TIME = 'program_begin_time'
    PROGRAM_END_TIME = 'program_end_time'
    CATEGORY_ID = 'category_id'


class CategoryField:
    ID = 'id'
    NAME = 'name'
    COLOR = 'color'


class Channel:
    def __init__(self, id, name, program, streamURL, iconURL, category, order, next_programs=None):
        self.id = id
        self.name = name
        self.program = program
        self.streamURL = streamURL
        self.iconURL = iconURL
        self.category = category
        self.order = order
        if next_programs is None:
            self.next_programs = []
        else:
            self.next_programs = next_programs

    def hasEPG(self):
        return self.program is not None and self.program.hasEPG()

    def getProgramByTime(self, timestamp):
        if not self.hasEPG():
            return None

        if self.program.beginTime <= timestamp < self.program.endTime:
            return self.program

        for program in self.next_programs:
            if program.beginTime <= timestamp < program.endTime:
                return program
        return None

    @staticmethod
    def from_smarty_response(ch, category, order):
        begin_time = ch[ChannelField.PROGRAM_BEGIN_TIME]
        end_time = ch[ChannelField.PROGRAM_END_TIME]
        id = ch[ChannelField.ID]
        icon = ch[ChannelField.ICON_URL]
        url = ch[ChannelField.STREAM_URL]
        name = ch[ChannelField.NAME]

        if 0 < begin_time < end_time:
            program_name = ch[ChannelField.PROGRAM_NAME]
            program_description = ch[ChannelField.PROGRAM_DESCRIPTION]
            program_id = ch.get('program_id', -1)
            program = Program(program_id, program_name, begin_time, end_time, program_description, id)
        else:
            program = None

        next_program_id = ch.get('next_program_id', -1)
        if next_program_id >= 0:
            next_begin_time = ch['next_program_begin_time']
            next_end_time = ch['next_program_end_time']
            next_program_name = ch['next_program_name']
            next_program_description = ch['next_program_description']
            next_program = Program(next_program_id, next_program_name, next_begin_time,
                                   next_end_time, next_program_description, id)
            next_programs = [next_program]
        else:
            next_programs = []

        return Channel(id, name, program, url, icon, category, order, next_programs)


class Category:
    DEFAULT_COLOR = '000000'

    def __init__(self, id, name, color):
        self.id = id
        self.name = name
        self.color = color

    @staticmethod
    def from_smarty_response(category_dict):
        id = category_dict[CategoryField.ID]
        name = category_dict[CategoryField.NAME]
        color = category_dict[CategoryField.COLOR]
        if len(color) < 6:
            color = Category.DEFAULT_COLOR
        return Category(id, name, color)


class ChannelAPIClient(BaseSmartyApiClient):
    def __init__(self, user=None):
        super(ChannelAPIClient, self).__init__(user)
        self._channels = {}
        self._categories = {}

    def refresh_categories(self):
        """
        Refresh categories list
        Returns error code
        """
        if not self._auth_ok:
            return BaseSmartyApiClient.NOT_AUTHORIZED

        categories_url = self._make_url_with_args(SmartyAPIURL.CATEGORY_LIST, {})
        response = self._request_json(categories_url)
        if response['error'] != 0:
            return response['error']
        self._categories = {}
        for cat in response['categories']:
            category = Category.from_smarty_response(cat)
            self._categories[category.id] = category
        return 0

    def get_category_by_id(self, category_id):
        assert isinstance(category_id, int)

        if category_id in self._categories:
            return self._categories[category_id]
        return None

    def get_channel_by_id(self, channel_id):
        assert isinstance(channel_id, int)

        if channel_id in self._channels:
            return self._channels[channel_id]
        return None

    def get_channels_by_category(self, category_id):
        if category_id is None:
            return []

        out = []
        for id in self._channels:
            if self._channels[id].category.id == category_id:
                out.append(self._channels[id])
        return out

    def refresh_channels(self, programs_limit=1):
        if not self._auth_ok:
            log_info("Can't refresh channels, user is not authorized", "iptv.refresh_channels")
            return BaseSmartyApiClient.NOT_AUTHORIZED

        params = dict()
        if self._user.account_sort:
            params['account_sort'] = self._user.account_sort
        params['programs_limit'] = programs_limit

        channels_url = self._make_url_with_args(SmartyAPIURL.CHANNEL_LIST, params)
        response = self._request_json(channels_url)
        if response['error'] != 0:
            return response['error']

        self._channels = {}
        order = 1
        for ch in response['channels']:
            category_id = ch['category_id']
            category = self.get_category_by_id(category_id) if self.categories is not None else None
            channel = Channel.from_smarty_response(ch, category, order)
            self._channels[channel.id] = channel
            order += 1
        return 0

    def get_channel_url(self, cid, direct_url=False):
        """
        This method doesn't check stream
        Returns stream URL
        """
        if direct_url and not self._auth_ok:
            return BaseSmartyApiClient.NOT_AUTHORIZED, None
        assert isinstance(cid, int)

        params = {
            'cid': cid,
            'redirect': 0 if direct_url else 1
        }

        channels_url = self._make_url_with_args(SmartyAPIURL.CHANNEL_URL, params)

        if not direct_url:
            return None, channels_url

        response = self._request_json(channels_url)
        return None, response["url"]

    def get_next_refresh_gap(self):
        now = int(time())
        inf = now + 86400  # one day

        gap_start_time = inf
        gap_end_time = inf

        for channel in self.channels:
            program = channel.program
            next_program = channel.next_programs[0] if len(channel.next_programs) > 0 else None

            if program is not None and program.beginTime != 0 and program.endTime > now:
                gap_start_time = min(gap_start_time, program.endTime)

            if next_program is not None and next_program.beginTime != 0:
                start_time = max(next_program.beginTime, now)
                gap_start_time = min(gap_start_time, start_time)
                gap_end_time = min(gap_end_time, next_program.endTime - 3)

        log_info("RefreshEvent", "%s %s" % (gap_start_time, gap_end_time))
        if gap_start_time >= inf or gap_end_time >= inf:
            # no programs found
            return now + 90, now + 90

        if gap_start_time < now + 90:
            gap_start_time = now + 90
        if gap_end_time < now + 1:
            gap_end_time = now + 90

        if gap_end_time < gap_start_time:
            gap_start_time = gap_end_time

        return gap_start_time, gap_end_time

    def get_next_program_end_time(self):
        end_time = int(time()) + 86400  # one day

        now = time()
        for channel in self.channels:
            program = channel.getProgramByTime(now)
            if program is None or program.beginTime == 0:
                continue
            end_time = min(end_time, program.endTime)
        return end_time

    @property
    def channels(self):
        if self._channels:
            return self._channels.values()
        return []

    @property
    def categories(self):
        if self._categories:
            return self._categories.values()
        return None
