from datetime import date

from smarty_base_api import BaseSmartyApiClient, SmartyAPIURL
from resources.lib.common import log_info, log_debug, log_error, log_warning


class Program:
    def __init__(self, id, name, beginTime, endTime, description, category='', channelId=-1):
        self.id = id
        self.name = name
        self.description = description
        self.beginTime = beginTime
        self.endTime = endTime
        self.meta = None
        self.shortName = u""
        self.rating = 0
        self.previewURL = ''
        self.category = ''
        self.has_record = False
        self.has_npvr_service = False
        self.channelId = channelId

    def hasEPG(self):
        return self.beginTime != 0 and self.beginTime < self.endTime

    @staticmethod
    def from_smarty_response(program_dict, channelId):
        id = program_dict.get('id', 0)
        name = program_dict.get('name', '')
        short_name = program_dict.get('short_name', '')
        preview_url = program_dict.get('preview', '')
        rating = program_dict.get('rating', 0)
        begin_time = program_dict.get('begin_time', 0)
        end_time = program_dict.get('end_time', 0)
        meta = program_dict.get('meta')
        description = program_dict.get('description', '')
        category = program_dict.get('category', '')
        has_record = program_dict.get('has_record', 0) == 1
        has_npvr_service = program_dict.get('has_npvr_service', 0) == 1

        program = Program(id, name, begin_time, end_time, description, category, channelId)
        program.previewURL = preview_url
        program.shortName = short_name
        program.rating = rating
        program.meta = meta
        program.has_record = has_record
        program.has_npvr_service = has_npvr_service
        return program


class ProgramAPIClient(BaseSmartyApiClient):
    def __init__(self, user=None):
        super(ProgramAPIClient, self).__init__(user)
        self._programs = {}

    def get_program_url(self, cid, time=None, pid=None, direct_url=False):
        if direct_url and not self._auth_ok:
            return BaseSmartyApiClient.NOT_AUTHORIZED, None
        assert isinstance(cid, int) and isinstance(time, int)

        params = {
            'cid': cid,
            'redirect': 0 if direct_url else 1,
        }
        if time:
            params['time'] = time
        elif pid is not None:
            params['pid'] = pid

        program_url = self._make_url_with_args(SmartyAPIURL.PROGRAM_URL, params)

        if not direct_url:
            return None, program_url
        response = self._request_json(program_url)
        return None, response["url"]

    def get_program_template_url(self, cid):
        params = {
            'cid': cid,
            'time': '%d',
        }
        return self._make_url_with_args(SmartyAPIURL.PROGRAM_URL, params)

    def get_program_by_time(self, cid, timestamp):
        ts_date = date.fromtimestamp(timestamp)
        programs = self.get_programs(cid, ts_date)
        if not programs:
            return None

        for program in programs:
            if program.beginTime <= timestamp < program.endTime:
                return program
        return None

    def get_program_by_id(self, cid, date, id):
        programs = self.get_programs(cid, date)
        if programs is None:
            return None
        for program in programs:
            if program.id == id:
                return program
        return None

    def _request_programs(self, cid, date_str):
        if not self._auth_ok:
            log_info("Can't get program, user is not authorized", "smartyApiClient.get_programs")
            return BaseSmartyApiClient.NOT_AUTHORIZED, None

        params = {
            'cid': cid,
            'date': date_str
        }

        url = self._make_url_with_args(SmartyAPIURL.PROGRAM_LIST, params)
        response = self._request_json(url)

        if response['error'] != 0:
            return response['error'], None
        return 0, response['programs']

    def _convert_programs_list(self, program_list, channelId):
        return [Program.from_smarty_response(program_dict, channelId) for program_dict in program_list]

    def get_programs(self, cid, date):
        date_str = date.strftime("%d.%m.%Y")
        if (date_str, cid) in self._programs:
            return self._programs[(date_str, cid)]

        error, programs = self._request_programs(cid, date_str)
        if error != 0:
            return None
        out = self._convert_programs_list(programs, cid)
        self._programs[(date_str, cid)] = out
        return out

    def get_programs_by_category(self, category_id):
        assert isinstance(category_id, int)

        if not self._auth_ok:
            log_info("Can't get programs by category, user is not authorized", "smartyApiClientget_programs_by_category")
            return BaseSmartyApiClient.NOT_AUTHORIZED, None

        params = {
            'category_id': category_id,
        }

        programs_url = self._make_url_with_args(SmartyAPIURL.PROGRAM_CATEGORY_CHANNEL_LIST, params)
        response = self._request_json(programs_url)

        if response['error'] != 0:
            return response['error'], None
        out = self._convert_programs_list(response['programs'], -1)
        return 0, out
